const GOODS_URI = require("../config.js").GOODS_URI;
const req = require("../utils/request").httpReq;

/**
 * 示例
 *  exports.方法名 = async ctx => {
 *    const url = `${设置好的域名}/向服务端发送的请求地址`;
 *    const opts = {
 *      url,
 *      method:"发送什么请求"，
 *      qs: ctx.request.query （get请求的参数）
 *      body：ctx.request.body （post请求的参数）
 *      json：boolean (是否转换为json)
 *    };
 *    ctx.body = await req(ctx,opts); 将发送请求后的数据传递给前端页面
 *  };
 */
// 获取分类
exports.getCategoryList = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/get_goods_category_list`;
    const opts = {
        url,
        method: "GET",
    };
    ctx.body = await req(ctx, opts);
};
// 获取商品列表
exports.getList = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/get_goods_list`;
    const opts = {
        url,
        method: "GET",
        qs: ctx.request.query,
    };
    ctx.body = await req(ctx, opts);
};
// 商品上架
exports.putOnline = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/online`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 商品下架
exports.putOffline = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/offline`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 设置商品上架
exports.putOnlineSite = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/online_site`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 获取商品详情列表
exports.getGoodsInfo = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/get_goods_info`;
    const opts = {
        url,
        method: "GET",
        qs: ctx.request.query,
    };
    ctx.body = await req(ctx, opts);
};
// 商品审核通过
exports.postSuccess = async (ctx) => {
    const opts = {
        url: `${GOODS_URI}/goods/background/audit_pass`,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 营销活动列表
exports.getMarketingList = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/marketing_list`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 检查商品名称是否被占用
exports.checkGoodsName = async (ctx) => {
    const opts = {
        url: `${GOODS_URI}/goods/background/check_goods_name`,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 获取查询活动列表
exports.getFindGoodsList = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/goods_list`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 审核拒绝
exports.auditReject = async (ctx) => {
    const opts = {
        url: `${GOODS_URI}/goods/background/audit_reject`,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};

// 添加活动列表
exports.getAddMarketingList = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/add_marketing`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 获取门店列表
exports.getShopsList = async (ctx) => {
    const opts = {
        url: `${GOODS_URI}/shop/background/shop_list`,
        method: "GET",
        qs: ctx.request.query,
    };
    ctx.body = await req(ctx, opts);
};
// 添加门店
exports.addShop = async (ctx) => {
    const opts = {
        url: `${GOODS_URI}/shop/background/add_shop`,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 查看列表详情
exports.getMarketingInfo = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/marketing_info`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 修改商品
exports.editGoods = async (ctx) => {
    const opts = {
        url: `${GOODS_URI}/goods/background/edit_goods`,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    ctx.body = await req(ctx, opts);
};
// 开启关闭状态
exports.updateMarketingList = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/update_marketing`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//团长列表
exports.coloneList = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/colonel_list`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//团长申请列表
exports.coloneApplyList = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/colonel_apply_list`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//团长申请操作
exports.colonelAudit = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/colonel_audit`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

// 商家列表
exports.otaList = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/ota_list`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

// 商家列表-add
exports.addOta = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/add_ota`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};
// 商家列表-禁用
exports.deleteOta = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/delete_ota`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};
// 商家列表-修改
exports.updateOta = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/update_ota`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};
//商家列表-取消禁用
exports.undeleteOta = async (ctx) => {
    const url = `${GOODS_URI}/goods/background/undelete_ota`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};
//团长获取自提点信息
exports.colonelApplyPoint = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/colonel_show_apply_point`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//团长添加自提点
exports.colonelAddApplyPoint = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/colonel_add_apply_point`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//营销中心-活动列表
exports.yingxiaoList = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/yingxiao_list`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//营销中心-活动列表
exports.toogleYingxiaoOnlineStatus = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/toogle_yingxiao_online_status`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//营销中心-充值
exports.addAmount = async (ctx) => {
    const url = `${GOODS_URI}/coupon/background/add_amount`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//营销中心-获取充值详情
exports.capitalPoolDetail = async (ctx) => {
    const url = `${GOODS_URI}/coupon/background/capital_pool_detail`;
    const opts = {
        url,
        method: "GET",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//营销中心-查看活动配置
exports.yingxiaoConfig = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/yingxiao_config`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};

//营销中心-修改活动配置
exports.yingxiaoSaveConfig = async (ctx) => {
    const url = `${GOODS_URI}/marketing/background/yingxiao_save_config`;
    const opts = {
        url,
        method: "POST",
        json: true,
        body: ctx.request.body,
    };
    const res = await req(ctx, opts);
    ctx.body = res;
};
